!module ootmap_mod
!  use global
!  implicit none
!!  real, dimension(12,64) :: rawcols
!   public ootcreate, ootsubtract, ootadd
!contains



  !---------------------------------------------------------------------
  subroutine ootraw(rawcols,datetime)

  !---------------------------------------------------------------------

    use global
    use dal
    use cal

    integer :: nevents,iccd,icol,iev, l1, irow
    real, dimension(12,64,200) :: rawcols
    integer, allocatable, dimension(:) :: ootx, ooty, ootpi, ootccd
    integer :: nccd,ccdmin,ccdmax,i
    character(80)   :: submode,instrume
    character(8), dimension(12) :: extnames
    character(20)                :: datetime
    integer, dimension(12) :: windowX0, windowY0, windowDx, windowDy
    real     :: ootfrac

    integer(kind=int16), dimension(:), pointer, save :: ip1, ip2, ip3
    integer(kind=int8), dimension(:), pointer, save :: ip4

    type(DataSetT) :: set
    type(TableT) :: tab,expos
    type(ColumnT) :: col(4)

      
    extnames(1) = "EXPOSU01"
    extnames(2) = "EXPOSU02"
    extnames(3) = "EXPOSU03"
    extnames(4) = "EXPOSU04"
    extnames(5) = "EXPOSU05"
    extnames(6) = "EXPOSU06"
    extnames(7) = "EXPOSU07"
    extnames(8) = "EXPOSU08"
    extnames(9) = "EXPOSU09"
    extnames(10) = "EXPOSU10"
    extnames(11) = "EXPOSU11"
    extnames(12) = "EXPOSU12"

! open list

    set = DataSet(ootlist, read) 
    tab = table(set, "EVENTS") 
    
    nevents =  numberOfRows(tab)

    allocate(ootx(nevents))
    allocate(ooty(nevents))
    allocate(ootpi(nevents))
    allocate(ootccd(nevents))

     
    if (hasAttribute(set,"INSTRUME")) then
      instrume =  stringAttribute(tab,"INSTRUME")   
    else
      call warning("MissingAttribute","Keyword INSTRUME is missing")
    endif  
    if (hasAttribute(set,"SUBMODE")) then
      submode =  stringAttribute(tab,"SUBMODE")   
    else
      call warning("MissingAttribute","Keyword SUBMODE is missing")   
    endif  
    if (hasAttribute(set,"DATE-OBS")) then
      datetime = stringAttribute(set,"DATE-OBS")
    else
      call warning("MissingAttribute","Keyword DATE-OBS is missing")   
    endif 

    if (index(instrume,"EPN") < 1) then
      call error ("FileMismatch","OOT list is not an EPN event list")
    endif

    if (submode .eq. "PrimeSmallWindow") then
       ccdmin=4
       ccdmax=4
       ootfrac=0.0116
     else if (submode .eq. "PrimeLargeWindow") then
       ccdmin=1
       ccdmax=12 
       ootfrac=0.0016      
     else if (submode .eq. "PrimeFullWindow") then
       ccdmin=1
       ccdmax=12 
       ootfrac=0.063 
     else if (submode .eq. "PrimeFullWindowExtended") then 
       ccdmin=1
       ccdmax=12
       ootfrac=0.023 
    else
       ccdmin=1
       ccdmax=12 
       ootfrac=0.063  
      call warning("UnknownSubmode",submode)
    endif   


    do nccd=ccdmin,ccdmax
      windowX0(nccd)=0
      windowY0(nccd)=0
      windowDx(nccd)=0
      windowDy(nccd)=0
      if (hasBlock(set,extnames(nccd))) then
        expos = table(set, extnames(nccd))
        if (hasAttribute(expos,"WINDOWX0")) then
          windowX0(nccd) =  int32Attribute(expos,"WINDOWX0") 
        else
          call warning("MissingAttribute","Keyword WINDOWX0 is missing")  
        endif    
        if (hasAttribute(expos,"WINDOWY0")) then
          windowY0(nccd) =  int32Attribute(expos,"WINDOWY0")  
        else
          call warning("MissingAttribute","Keyword WINDOWY0 is missing") 
        endif  
        if (hasAttribute(expos,"WINDOWDX")) then
          windowDx(nccd) =  int32Attribute(expos,"WINDOWDX")
        else
          call warning("MissingAttribute","Keyword WINDOWDX is missing")
        endif  
        if (hasAttribute(expos,"WINDOWDY")) then
          windowDy(nccd) =  int32Attribute(expos,"WINDOWDY")   
        else
          call warning("MissingAttribute","Keyword WINDOWDY is missing")
        endif 
      else
        call warning("MissingBlock","Exposure extension "//extnames(nccd)//" not found")
      endif
    enddo
   

    l1=1
    col(l1) = column(tab,"RAWX",read)  
    ip1 => int16Data(col(l1))
    l1=l1+1 
    col(l1) = column(tab,"RAWY",read)  
    ip2 => int16Data(col(l1))
    l1=l1+1 
    col(l1) = column(tab,"PI",read)  
    ip3 => int16Data(col(l1))
    l1=l1+1 
    col(l1) = column(tab,"CCDNR",read)
    ip4 => int8Data(col(l1))


    do i=0,nevents-1
      ootx(i+1) = ip1(i)
      ooty(i+1) = ip2(i)
      ootpi(i+1) = ip3(i)
      ootccd(i+1) = ip4(i)
    end do

  do iccd=1,12
!    if  (windowDy(iccd) .gt. 0) then
       do icol=1,64
         do irow=1,200
           rawcols(iccd,icol,irow)=0.0 
         enddo
       enddo 
!    endif 
  enddo
 

  do iev=1,nevents
    if (windowDy(ootccd(iev)) .gt. 0) then
      if (ootpi(iev) .ge. pimin .and. ootpi(iev) .le. pimax) then 
        do irow=windowY0(ootccd(iev)),windowY0(ootccd(iev))+windowDy(ootccd(iev))-1
          rawcols(ootccd(iev),ootx(iev),irow)=rawcols(ootccd(iev),ootx(iev),irow)+ootfrac/float(windowDy(ootccd(iev)))
        enddo
      endif
    endif 
  enddo  


  end subroutine ootraw



 
  !----------------------------------------------------------------------
  subroutine ootsubtract
  !---------------------------------------------------------------------- 

    use global
    use dal
    use cal

  integer :: nx,ny,ix,iy

  nx=imsize(1)
  ny=imsize(2)   


  do ix=1,nx
    do iy=1,ny
      ifeld(ix,iy)=ifeld(ix,iy)-oot_sky(ix,iy)
    enddo
  enddo


  end  subroutine ootsubtract


 
  !----------------------------------------------------------------------
  subroutine ootadd
  !---------------------------------------------------------------------- 

    use global
    use dal
    use cal

  integer :: nx,ny,ix,iy

  nx=imsize(1)
  ny=imsize(2)   


  do ix=1,nx
    do iy=1,ny
      iback(ix,iy)=iback(ix,iy)+oot_sky(ix,iy)
    enddo
  enddo


  end  subroutine ootadd




  !----------------------------------------------------------------------
  subroutine ootcreate
  !----------------------------------------------------------------------

    use global
    use dal
    use cal

  integer :: nx,ny,xpix,ypix,ix,iy,ierr,nsub,xsub,ysub
  integer , dimension(:), allocatable :: chipix,chipiy,ccdinr
  integer(kind=int32) :: pixnum,rownum,irow
  real(kind=double) :: pxareaconv,sum,nsubsq
  real(kind=double) :: cos_pa, sin_pa, xr, yr, dra, ddec, r0r, d0r,rr, dr
  real(kind=double), dimension(:), allocatable :: rapix,decpix 
  character(len=20)  :: datetime 
  real, dimension(12,64,200) :: rawcols
!!! IT 2012/07
  real(kind=double) :: focalLength
!  real(kind=double), parameter :: pi=3.141592654
  real(kind=double), parameter :: pi=3.14159265358979323846d0
  real(kind=double), parameter :: pi180=pi/180d0, overpi180=180d0/pi


  nx=imsize(1)
  ny=imsize(2)  

  nsub=2
  nsubsq=float(nsub*nsub)

  pixnum=nx*ny*nsub*nsub
  rownum=ny*nsub*nsub
  allocate(rapix(rownum))
  allocate(decpix(rownum))
  allocate(chipix(rownum))
  allocate(chipiy(rownum))
  allocate(ccdinr(rownum))

!  allocate(ratest(pixnum))
!  allocate(dectest(pixnum))
!  allocate(xtest(pixnum))
!  allocate(ytest(pixnum))
!  allocate(ccdtest(pixnum))



  allocate(oot_sky(nx,ny))
 
!!! IT 2012/07
!!! wrong and not used:
!  areascale=crdelt1*3600.D0/4.D0*crdelt2*3600.D0/4.D0
!!! convert raw pixels to image pixel: conversion factor with which the flux
!!! has to be multiplied:
!!! size of image pixel in arcsec: crdeltI * 3600.
!!! conversion arcsec to mm:     * focalLength * pi/180. / 3600.
! get focal length of telescope and pixel sizes in mm from CAL
  call CAL_setState(instrument=EPN)
  call CAL_getMiscellaneousData('FOCAL_LENGTH', focalLength)
! conversion factor
  pxareaconv = &
! x pixel size
       dabs(crdelt1) * focalLength * pi180 / &
       CAL_getMiscellaneousDataValue('MM_PER_PIXEL_X') &
       * &
! y pixel size
       dabs(crdelt2) * focalLength * pi180 / &
       CAL_getMiscellaneousDataValue('MM_PER_PIXEL_Y')


  call ootraw(rawcols,datetime)


  do xpix = 1,nx
    irow=0
    do ypix = 1,ny
      do xsub=1,nsub
        do ysub=1,nsub
          irow=irow+1
          cos_pa = dcos(crota2 * pi180)
          sin_pa = dsin(crota2 * pi180)
          xr = (xpix-crpix1-0.5+(2.*xsub-1.)/(2.*nsub)) * crdelt1 * pi180
          yr = (ypix-crpix2-0.5+(2.*ysub-1.)/(2.*nsub)) * crdelt2 * pi180
          dra =  xr * cos_pa - yr * sin_pa
          ddec = xr * sin_pa + yr * cos_pa
          r0r = crval1 * pi180
          d0r = crval2 * pi180
          call xy2radec(dra,ddec,r0r,d0r,rr,dr)
          rapix(irow) = rr * overpi180
          decpix(irow) = dr *overpi180
        end do
      end do
    end do
    call sky2raw(rownum,rapix,decpix,datetime,chipix,chipiy,ccdinr)
    irow=0
    do ypix = 1,ny
      sum=0.0
      do xsub=1,nsub
        do ysub=1,nsub
           irow=irow+1

           if (ccdinr(irow)>0 .and. (chipix(irow)>0 .and.   chipix(irow)<=64  &
               .and. chipiy(irow)>0  .and.  chipiy(irow)<=200 )) then
             sum=sum+rawcols(ccdinr(irow),chipix(irow),chipiy(irow))
           endif
        end do
      end do
      oot_sky(xpix,ypix)=sum/nsubsq * pxareaconv
    end do   
  end do 


  deallocate(rapix)
  deallocate(decpix)
  deallocate(chipix)
  deallocate(chipiy)
  deallocate(ccdinr)



  end subroutine ootcreate 
    



  !----------------------------------------------------------------------
  subroutine sky2raw(pixnum,ra,dec,datetime, &
                       chipix,chipiy,ccdinr)
  !----------------------------------------------------------------------

    use global
    use dal
    use cal

    integer(kind=int32) :: pixnum,ipix
    real(kind=double),  dimension(pixnum), intent(in) :: ra, dec
!    real(kind=double), intent(in) :: ra_pnt,dec_pnt,pa_pnt
    character(len=20), intent(in) :: datetime
    integer , dimension(pixnum), intent(out) :: chipix, chipiy, ccdinr
    
    real(kind=double), dimension(:), pointer :: theta, phi, &
!    real(kind=single), dimension(:), pointer :: theta, phi, &
                       cc2x, cc2y, cc2z, cc1x, cc1y, cc1z      
    real(kind=double) :: timestamp
    integer(int16), dimension(:), pointer :: chipX, chipY
    integer(int8), dimension(:), pointer :: ccdnr  
    integer :: i, j, j0

    real(kind=double), parameter :: pi180=3.14159265358979323846d0/180d0

    type(SpacecraftAttitudeType) :: myattitude  
    type (BoreSightT) :: boresight
    type(EquatorialDirectionType), allocatable:: srcDirs(:)


     allocate(srcdirs(pixnum))

    do ipix=1,pixnum
      srcDirs(ipix)%ra = ra(ipix)*pi180
      srcDirs(ipix)%dec = dec(ipix)*pi180
    end do


    timeStamp = toTimeTag(datetime) ! in caloalutils/caloaldefs.f90        
    boresight = CAL_getBoresight(TimeStamp)


    myattitude%ra=ra_pnt*pi180
    myattitude%dec=dec_pnt*pi180
    myattitude%apos=pa_pnt*pi180
    call CAL_setState(EPN)
    call CAL_withSacCoordOrientation(boresight, myAttitude)
    call CAL_projectOntoTelCoord(boresight, srcDirs, theta, phi)
    call CAL_telCoordToCamCoord2(theta, phi, cc2x, cc2y, cc2z)
    call CAL_camCoord2ToCamCoord1(cc2x, cc2y, cc2z, cc1x, cc1y, cc1z)
    call CAL_camCoord1ToChipCoord(cc1x, cc1y, cc1z, chipX, chipY, ccdNr)

    do ipix=1,pixnum
      chipix(ipix)=chipX(ipix-1)
      chipiy(ipix)=chipY(ipix-1)
      ccdinr(ipix)=ccdNr(ipix-1)
    end do
 
    deallocate(srcdirs)
   
  end subroutine sky2raw



!end module
